package JCAT;

import java.io.*;
import java.awt.*;


class Printer extends Thread
{
    TextArea win_;
    InputStream stream_;

    Printer(TextArea win, InputStream stream)
    {
	win_    = win;
	stream_ = stream;
    }

    public void run()
    {
	byte b [] = new byte [10240];
	int len;

	while (true) {
	    try {
		len = stream_.read(b, 0, 10240);
	    } catch (IOException e) {
		win_.appendText(e.getMessage());
		return;
	    }

	    if (len == -1)
		break;

	    win_.appendText(new String(b, 0, 0, len));
	}
    }
}


class Builder implements Commands
{
    private String model_;
    private String j2sDir;
    private String bashDir;

    private String j2sOpt;

    private TextArea outText_;
    private TextArea errText_;
    private Label label_;

    private Runtime rt_;

    private Process j2sProc;

    private Printer out_, err_;

    Builder(String j2sdir, String bashdir, String j2sopt, 
	    String m, TextArea out, TextArea err, Label l)
    {
	j2sDir  = j2sdir;
	bashDir = bashdir;
	j2sOpt  = j2sopt;
	model_  = m;

	outText_= out;
	errText_= err;

	label_  = l;

	rt_     = Runtime.getRuntime();
	j2sProc = null;

	out_    = null;
	err_    = null;
    }
  
    public boolean doFormalModel()
    {
	/*
	String [] cmd = {bashDir + "/" + bash_, " -c ",
			 j2sDir + "/" + j2s_ + " -o " + model_ + " " + j2sOpt};
	*/

	String cmd = j2sDir + "/" + j2s_ + " -o " + model_ + " " + j2sOpt;

	label_.setText("Building formal model ...");
	
	System.out.println(cmd);

	try {
	    j2sProc = rt_.exec(cmd);
	} catch(Exception e) {
	    errText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");
	    return false;
	}
    
	out_ = new Printer(outText_, j2sProc.getInputStream());
	err_ = new Printer(errText_, j2sProc.getErrorStream());
    
	boolean errors = false;

	out_.start();
	err_.start();

	try {
	    if (j2sProc.waitFor() != 0)
		errors = true;
	} catch(InterruptedException e) {
	    errText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");
	    return false;
	}

	try {
	    out_.join();
	    err_.join();
	} catch (InterruptedException e) {
	    outText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");
	    return false;
	}
    
	if (errors) {
	    label_.setText("Errors");
	    return false;
	}

	return true;
    }
}


class Analysis implements Commands
{
    public static final int INVALID_ENDSTATE = 1, ASSERTION_VIOLATED = 2;

    private int state_;

    // Directories
    private String spinDir, ccDir, timeDir;

    // Options
    private String spinOpt, ccOpt, panOpt;

    private String model_;

    private TextArea outText_;
    private TextArea errText_;
 
    private Label label_;
    private Runtime rt_;

    private Builder build_;

    private Process current_process;

    Analysis(DirBox dirBox, String j2sopt, String pcopt, String peopt, 
	     TextArea out, TextArea err, Label l)
    {
	spinDir    = dirBox.getSpinDir();
	ccDir      = dirBox.getCCDir();
	timeDir    = dirBox.getTimeDir();

	ccOpt      = pcopt;
	panOpt     = peopt;

	model_     = dirBox.getModel();
	spinOpt    = "-a " + model_;

	outText_   = out;
	errText_   = err;

	label_     = l;
	rt_        = Runtime.getRuntime(); 

	build_     = new Builder(dirBox.getJ2SDir(), dirBox.getBashDir(), 
				 j2sopt, dirBox.getModel(), out, err, l);

	current_process = null;
    }


    public int getResult()
    {
	if (!build_.doFormalModel())
	    return 0;

	String spin_cmd   = spinDir + "/" + spin_ + " " + spinOpt;
	String cc_cmd     = ccDir + "/" + cc_ + " " + ccOpt + " pan.c -o pan";
	String pan_cmd    = (Configure.timeAvail ? timeDir + "/" + time_ : "")
	    + " " + pan_ + " " + panOpt;
    
	Process spin_proc;
	Process cc_proc;
	Process pan_proc;
   
	label_.setText("Building analyzer ...");
   
	System.out.println("\n" + spin_cmd);

	try {
	    current_process = spin_proc = rt_.exec(spin_cmd);
	} catch(Exception e) {
	    errText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");
	    return 0;
	}      
          
	try {
	    spin_proc.waitFor();
	    current_process = null;
	} catch(InterruptedException e) {
	    errText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");
	    return 0;
	}
      

	label_.setText("Compiling analyzer ...");

	System.out.println("\n" + cc_cmd);

	try {
	    current_process = cc_proc = rt_.exec(cc_cmd);
	} catch(Exception e) {
	    errText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");	  

	    System.out.println("Exception1: " + e.getMessage());

	    return 0;
	}
      
	try {
	    cc_proc.waitFor();
	    current_process = null;
	} catch(InterruptedException e) {
	    errText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");

	    System.out.println("Exception2: " + e.getMessage());

	    return 0;
	}

	label_.setText("Running analyzer ...");

	System.out.println("\n" + pan_cmd);

	try {
	    current_process = pan_proc = rt_.exec(pan_cmd);
	} catch(Exception e) {
	    errText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");
	    return 0;
	}

	OutLex scanner = new OutLex(pan_proc.getInputStream());

	Printer err = new Printer(errText_, pan_proc.getErrorStream());

	if (Configure.timeAvail)
	    errText_.appendText("\n --- Time statistics ---\n");

	err.start();
    
	try {
	    scanner.yylex();
	} catch (Exception e) {
	    errText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");
	    return 0;
	}

	try {
	    err.join();
	} catch (InterruptedException e) {
	    errText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");
	    return 0;
	}
    
	try {
	    pan_proc.waitFor();
	    current_process = null;
	} catch (InterruptedException e) {
	    errText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");
	    return 0;
	}
    
	SpinOut spinOut = scanner.getOutput();

	label_.setText("Errors: " + spinOut.getErrors());
    
	outText_.appendText("\n --- Analysis statistics ---\n");
	spinOut.print(outText_);

	if (spinOut.getEndState()) 
	    return INVALID_ENDSTATE;

	if (spinOut.getAssertion()) 
	    return ASSERTION_VIOLATED;

	if (spinOut.outOfMemory())
	    label_.setText("OUT OF MEMORY!");

	return 0;
    }
}



