package JCAT;
import java.util.Vector;
import java.awt.*;


interface LineContainer
{
    public void initLine();
    public void addLine(SpinTrailLine line);
    public SpinTrailLine nextLine();
}


class LineBuffer implements LineContainer
{
    private final int size = 64;
    private SpinTrailLine buffer [];

    private int putPtr;
    private int takePtr;
    private int usedSlots;


    public LineBuffer()
    {
	buffer = new SpinTrailLine [size];
	putPtr = 0;
	takePtr = 0;
	usedSlots = 0; 
    }

  
    public synchronized void addLine(SpinTrailLine line)
    {
	while (usedSlots == size)
	    try {
		wait();
	    } catch (InterruptedException e) {
		return;
	    }

	buffer [putPtr] = line;
	putPtr = (putPtr + 1) % size;

	if (usedSlots ++ == 0)
	    notifyAll();
    }


    public synchronized SpinTrailLine nextLine()
    {
	while (usedSlots == 0)
	    try {
		wait();
	    } catch(InterruptedException e) {
		return null;
	    }

	SpinTrailLine line = buffer [takePtr];
	buffer [takePtr] = null;
	takePtr = (takePtr + 1) % size;

	if (usedSlots -- == size)
	    notifyAll();

	return line;
    }


    public synchronized void initLine()
    {
	putPtr = 0;
	takePtr = 0;
	usedSlots = 0;
    }
}


class LineVector implements LineContainer
{
    private Vector vector;
    private int next;


    public LineVector()
    {
	vector = new Vector();
	next = 0;
    }


    public void addLine(SpinTrailLine line)
    {
	vector.addElement(line);
    }


    public SpinTrailLine nextLine()
    {
	return (SpinTrailLine)vector.elementAt(next ++);
    }


    public void initLine()
    {
	synchronized (vector) {
	    next = 0;
	}
    }
}


class SpinTrailLine
{
    public static final int PROC_CREATE = 0;
    public static final int PROC_SEQ = 1; 
    public static final int PROC_TERMINATES = 2;
    public static final int TIMEOUT = 3;
    public static final int END_OF_TRAIL = 4;
    public static final int NO_NUMBER = 5;

    private int tag;

    private Integer step;
    private Integer procNumber;
    private String  procName;
    private Integer line;
    private Integer state;

    private Integer createNumber;
    private String createName;

    private String javaFile;
    private Integer javaLine;


    SpinTrailLine(String s)
    {
	step = new Integer(s);
    }

    SpinTrailLine(int t)
    {
	tag = t;
    }

    public void setTag(int t)
    {
	tag = t;
    }

    public int getTag()
    {
	return tag;
    }

    public int getStep()
    {
	return step.intValue();
    }


    public void setProcNumber(String s)
    {
	try {
	    procNumber = new Integer(s);
	} catch (NumberFormatException e) {
	    procNumber = new Integer(-1);
	}
    }

    public int getProcNumber()
    {
	return procNumber.intValue();
    }

    public void setProcName(String s)
    {
	procName = new String(s);
    }

    public String getProcName()
    {
	return procName;
    }

    public void setLineNo(String s)
    {
	line = new Integer(s);
    }

    public int getLineNo()
    {
	return line.intValue();
    }

    public void setState(String s)
    {
	state = new Integer(s);
    }

    public int getState()
    {
	return state.intValue();
    }

    public void setCreateNumber(String s)
    {
	createNumber = new Integer(s);
    }

    public int getCreateNumber()
    {
	return createNumber.intValue();
    }

    public void setCreateName(String s)
    {
	createName = new String(s);
    }

    public String getCreateName()
    {
	return createName;
    }

    public void setJavaFile(String file)
    {
	javaFile = new String(file);
    }

    public String getJavaFile()
    {
	return javaFile;
    }

    public void setJavaLine(Integer line)
    {
	javaLine = new Integer(line.intValue());
    }

    public int getJavaLine()
    {
	return javaLine.intValue();
    }
}


class SpinTrail
{
    private LineContainer lineCont;
  
    private Integer procNo;
    private Relation rel_;


    SpinTrail(LineContainer c)
    {
	lineCont = c;
    }


    public void addLine(SpinTrailLine line)
    {
	lineCont.addLine(line);
    }


    public void setRelation(Relation r)
    {
	rel_ = r;
    }


    public void initLine()
    {
	lineCont.initLine();
    }
  

    public SpinTrailLine nextLine()
    {
	SpinTrailLine line = lineCont.nextLine();

	if (line.getTag() != SpinTrailLine.PROC_SEQ)
	    return line;

	RelationLine rl = rel_.search(line.getLineNo());

	if (rl == null)
	    return new SpinTrailLine(SpinTrailLine.NO_NUMBER);

	line.setJavaFile(rl.getJavaFile());
	line.setJavaLine(rl.getJavaLine());

	return line;
    }    
}


class RelationLine 
{
    private int spinLine;
    private String javaFile;
    private Integer javaLine;

    public RelationLine(int line)
    {
	spinLine = line;
    }

    public int getLineNo()
    {
	return spinLine;
    }

    public void setJavaFile(String file)
    {
	javaFile = new String(file);
    }

    public String getJavaFile()
    {
	return javaFile;
    }

    public void setJavaLine(String line)
    {
	javaLine = new Integer(line);
    }

    public Integer getJavaLine()
    {
	return javaLine;
    }
}


class Relation
{
    private Vector lineVect;


    public Relation()
    {
	lineVect = new Vector();
    }


    public void addLine(RelationLine line)
    {
	lineVect.addElement((Object)line);
    }


    public RelationLine search(int key)
    {
	int lo = 0;
	int hi = lineVect.size();
	int med;

	RelationLine rl;

	while (lo < hi - 1) {
	    med = (lo + hi) / 2;
	    rl  = (RelationLine)lineVect.elementAt(med);

	    if (key == rl.getLineNo())
		return rl;

	    if (key < rl.getLineNo())
		hi = med;
	    else 
		if (key > rl.getLineNo())
		    lo = med;
	}

	return (lo < lineVect.size()) ? (RelationLine)lineVect.elementAt(lo):null;
    }
}


class TrailReader extends Thread
{
    TrailLex trailScan;
    Process  spinProc;

    TextArea text_;
    Label label_;


    TrailReader(String spinCmd, LineContainer c, TextArea t, Label l)
    {
	text_ = t;
	label_ = l;

	try {
	    spinProc = Runtime.getRuntime().exec("nice " + spinCmd);
	} catch (Exception e) {
	    synchronized (text_) {
		text_.appendText(e.getMessage() + "\n");
	    }
	    return;
	}

	trailScan = new TrailLex(spinProc.getInputStream(), c);

	setPriority(MAX_PRIORITY);
    }


    public void run()
    {
	try {
	    trailScan.yylex();
	} catch (Exception e) {
	    text_.appendText(e.getMessage() + "\n");
	    return;
	}

	try {
	    spinProc.waitFor();
	} catch (InterruptedException e) {
	    text_.appendText(e.getMessage() + "\n");
	    return;
	}
    }


    public SpinTrail getTrail()
    {
	return trailScan.getTrail();
    }


    public void quit()
    {
	if (spinProc != null)
	    spinProc.destroy();

	stop();
    }
}


