package JCAT;

import java.awt.*;
import java.io.*;
import java.util.*;


interface Traceable {
    public void makeTrail();
    public boolean nextStep();
    public void init();
    public void clear();
    public void quit();
}


class Debug implements Traceable, Commands
{
    private String spinOpt     = "-pt";

    protected String spinDir;
    protected String model_;

    protected TextArea outText_;
    protected TextArea errText_;

    protected Label label_;

    protected Runtime rt_;

    protected TrailReader trailRead;
    protected SpinTrail trail_;

    protected Vector frameVect;

    protected Hashtable fileHash;

    public Debug(DirBox dirBox, TextArea out, TextArea err, Label label) 
    {
	spinDir   = dirBox.getSpinDir();
	model_    = dirBox.getModel();
    
	outText_  = out;
	errText_  = err;

	label_    = label;
	rt_       = Runtime.getRuntime();
    
	frameVect = new Vector();
	fileHash  = new Hashtable();
    }

    public void makeTrail()
    {
	trailRead = new TrailReader(spinDir + "/" + spin_ + " " + spinOpt + " " + 
				    model_, new LineVector(), errText_, label_);

	trail_ = trailRead.getTrail();

	label_.setText("Generating trail ...");

	trailRead.start();

	FileInputStream proStream;

	try {
	    proStream = new FileInputStream(model_);
	} catch (IOException e) {
	    errText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");
	    return;
	}

	ProLex proScan = new ProLex(proStream);

	try {
	    proScan.yylex();
	} catch (Exception e) {
	    errText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");
	    return;
	}

	trail_.setRelation(proScan.getRelation());

	label_.setText("Press STEP to advance");
    }

    public void init()
    {
	for (int i = 0; i < frameVect.size(); i ++) {
	    ((ThreadFrame)frameVect.elementAt(i)).hide();
	    ((ThreadFrame)frameVect.elementAt(i)).dispose();
	}

	frameVect = new Vector();
	trail_.initLine();
	label_.setText("Press STEP to advance");
    }

    public void clear()
    {
	for (int i = 0; i < frameVect.size(); i ++) {
	    ((ThreadFrame)frameVect.elementAt(i)).hide();
	    ((ThreadFrame)frameVect.elementAt(i)).dispose();
	}

	frameVect = null;
    }

    public void quit()
    {
	if (trailRead != null)
	    trailRead.quit();

	trailRead = null;
	trail_ = null;

	if (frameVect == null)
	    return;
    
	for (int i = 0; i < frameVect.size(); i ++) {
	    ((ThreadFrame)frameVect.elementAt(i)).hide();
	    ((ThreadFrame)frameVect.elementAt(i)).dispose();
	}

	frameVect = null;
    }

    public boolean nextStep()
    {
	for(;;) {
	    SpinTrailLine line = trail_.nextLine();

	    switch (line.getTag())
		{
		case SpinTrailLine.NO_NUMBER:
		    {
			label_.setText("No line number information");
			return true;
		    }

		case SpinTrailLine.END_OF_TRAIL:
		    {
			label_.setText("End of trail");
			return false;
		    }

		case SpinTrailLine.PROC_CREATE:
		    {
			label_.setText("Thread (" + line.getProcNumber() + ") creates (" +
				       line.getCreateNumber() + ")");      
			frameVect.addElement(new ThreadFrame());
	    
			int index = line.getCreateNumber();
			ThreadFrame f = (ThreadFrame)frameVect.elementAt(index);

			f.setTitle("Thread " + index + " (" + line.getCreateName() + ")");
			f.show();
			return true;
		    }

		case SpinTrailLine.PROC_TERMINATES:
		    {
			label_.setText("Thread (" + line.getProcNumber() + ") terminates");
			return true;
		    }

		case SpinTrailLine.TIMEOUT:
		    {
			label_.setText("TIMEOUT!");
			return true;
		    }
	
		case SpinTrailLine.PROC_SEQ:
		    {
			label_.setText("Thread (" + line.getProcNumber() + "): " +
				       line.getJavaFile() + " line " + line.getJavaLine());
	    
			String file = line.getJavaFile();
			int no      = line.getJavaLine();
			int index   = line.getProcNumber();
	    
			ThreadFrame frame = (ThreadFrame)frameVect.elementAt(index);
			FileLoader fileLoad = (FileLoader)fileHash.get(file);
	
			if (fileLoad == null) {
			    label_.setText("Loading " + file + " ...");
	      
			    try {
				fileLoad = new FileLoader(file);
				fileLoad.load();
			    } catch (IOException e) {
				errText_.appendText("I/O ERROR:" + e.getMessage() + "\n");
				label_.setText("Ready");
				return false;
			    }

			    fileHash.put(file, fileLoad);
	      
			    label_.setText("Ready");
			}

			if (frame.refresh(fileLoad, no))
			    return true;
		    }
		}
	}
    }
}


class Simulation extends Debug
{
    private String spinOpt = "-p";
    private Builder build_;


    Simulation(DirBox dirBox, String j2sopt, TextArea out, TextArea err, Label l)
    {
	super(dirBox, out, err, l);
	build_  = new Builder(dirBox.getJ2SDir(), dirBox.getBashDir(), 
			      j2sopt, dirBox.getModel(), out, err, l);
    }


    public void makeTrail()
    {
	if (!build_.doFormalModel())
	    return;

	trailRead = new TrailReader(spinDir + "/" + spin_ + " " + spinOpt + " " + 
				    model_, new LineBuffer(), errText_, label_);

	trail_ = trailRead.getTrail();

	label_.setText("Generating trail ...");

	trailRead.start();

	FileInputStream proStream;

	try {
	    proStream = new FileInputStream(model_);
	} catch (IOException e) {
	    errText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");
	    return;
	}

	ProLex proScan = new ProLex(proStream);

	try {
	    proScan.yylex();
	} catch (Exception e) {
	    errText_.appendText(e.getMessage() + "\n");
	    label_.setText("Ready");
	    return;
	}

	trail_.setRelation(proScan.getRelation());

	label_.setText("Press STEP to advance");
    }


    public void init()
    {
	super.init();
	makeTrail();
    }
}
